@@ -21,9 +21,9 @@ use App::Multigit::Loop qw(loop);
 use Exporter 'import';
 
 our @EXPORT_OK = qw/
-    mgconfig mg_parent all_repositories 
+    mgconfig mg_parent 
+    all_repositories selected_repositories 
     base_branch set_base_branch mg_each
-    extract_future
 /;
 
 =head1 NAME
@@ -33,7 +33,7 @@ deal with git subrepositories.
 
 =cut
 
-our $VERSION = '0.09';
+our $VERSION = '0.10';
 
 =head1 PACKAGE VARS
 
@@ -87,6 +87,22 @@ our %BEHAVIOUR = (
     skip_readonly       => !!$ENV{MG_SKIP_READONLY},
 );
 
+=head2 @SELECTED_REPOS
+
+If this is not empty, it should contain paths to repositories. Relative paths
+will be determined relative to L<C<<mg_root>>|/mg_root>.
+
+Instead of using the C<.mgconfig>, the directories in here will be used as the
+list of repositories on which to work.
+
+Each repository's C<origin> remote will be interrogated. If this exists in the
+C<.mgconfig> then it will be used as normal; otherwise, it will be treated as
+though it had the default configuration.
+
+=cut
+
+our @SELECTED_REPOS;
+
 =head1 FUNCTIONS
 
 These are not currently exported.
@@ -105,12 +121,19 @@ sub mgconfig() {
 =head2 mg_parent
 
 Tries to find the closest directory with an C<mgconfig> in it. Dies if there is
-no mgconfig here.
+no mgconfig here. Optionally accepts the directory to start with.
 
 =cut
 
 sub mg_parent {
-    my $pwd = shift // dir->absolute;
+    my $pwd;
+    if (@_) {
+        $pwd = dir(shift);
+    }
+    else {
+        $pwd = dir;
+    }
+    $pwd = $pwd->absolute;
 
     PARENT: {
         do {
@@ -146,6 +169,54 @@ sub all_repositories {
     return $cfg;
 }
 
+=head2 selected_repositories
+
+This returns the repository configuration as determined by 
+L<C<<@SELECTED_REPOS>>|/@SELECTED_REPOS>. Directories that exist in the main
+config (L<all_repositories>) will have their configuration honoured, but unknown
+directories will have default configuration.
+
+=cut
+
+sub selected_repositories {
+    my $all_repositories = all_repositories;
+
+    return $all_repositories unless @SELECTED_REPOS;
+
+    my $bydir = +{ map {$_->{dir} => $_} values %$all_repositories };
+
+    my $selected_repos = {};
+
+    my $parent = mg_parent;
+
+    for my $dir (@SELECTED_REPOS) {
+        # Allow people to not have to worry about extracting blanks
+        next if not $dir;
+
+        $dir = dir($dir)->relative($parent);
+        if (exists $bydir->{$dir}) {
+            $selected_repos->{ $bydir->{$dir}->{url} } = $bydir->{$dir};
+        }
+        else {
+            my $url = 
+                try {
+                    _sensible_remote_url($dir);
+                }
+                catch {
+                    warn $_;
+                }
+            or next;
+
+            $selected_repos->{ $url } = {
+                url => $url,
+                dir => $dir,
+            }
+        }
+    }
+
+    return $selected_repos;
+}
+
 =head2 each($command)
 
 For each configured repository, C<$command> will be run. Each command is run in
@@ -323,7 +394,7 @@ form is used.
 
 sub each {
     my $command = shift;
-    my $repos = all_repositories;
+    my $repos = selected_repositories;
 
     my $f = fmap { _run_in_repo($command, $_[0], $repos->{$_[0]}) } 
         foreach => [ keys %$repos ],
@@ -387,29 +458,40 @@ sub init {
     } catch {};
 
     for my $dir (@dirs) {
-        my ($remotes) = capture {
-            system qw(git -C), $dir, qw(remote -v)
-                and return;
-        };
-
-        # FIXME: This seems fragile
-        next if $?;
-
-        if (not $remotes) {
-            warn "No remotes configured for $dir\n";
-            next;
-        }
-        my ($first_remote) = split /\n/, $remotes;
-        my ($name, $url) = split ' ', $first_remote;
-
+        my $url = try {
+                _sensible_remote_url($dir);
+            }
+            catch {
+                warn $_;
+            }
+        or next;
         $config{$url}->{dir} = $dir;
     }
-
     
     my $config_filename = dir($workdir)->file(mgconfig);
     Config::INI::Writer->write_file(\%config, $config_filename);
 }
 
+# Fetch either origin URL, or any URL. Dies if none.
+sub _sensible_remote_url {
+    my $dir = shift;
+    my ($remotes, $stderr, $exitcode) = capture {
+        system qw(git -C), $dir, qw(remote -v)
+            and return;
+    };
+
+    die $stderr if $exitcode;
+
+    if (not $remotes) {
+        die "No remotes configured for $dir\n";
+    }
+
+    my @remotes = split /\n/, $remotes;
+    my %remotes = map {split ' '} @remotes;
+
+    return $remotes{origin} // $remotes{ (keys %remotes)[0] }
+}
+
 =head2 base_branch
 
 Returns the branch that the base repository is on -the repository that contains
