@@ -5,7 +5,7 @@ use warnings;
 
 use base qw( IO::Async::Future );
 
-our $VERSION = '0.09';
+our $VERSION = '0.10';
 
 =head1 NAME
 
@@ -43,8 +43,9 @@ sub finally {
         my $f = shift;
         my @result;
 
+
         if (@result = $f->failure) {
-            @result = @result[2, $#result];
+            @result = @result[2 .. $#result];
         }
         else {
             @result = $f->get;
